structure GdkGeometry :>
  GDK_GEOMETRY
    where type t = GdkGeometryRecord.t
    where type gravity_t = GdkGravity.t =
  struct
    local
      open PolyMLFFI
    in
      val minWidth'Offset_ = call (getSymbol "giraffe_gdk_geometry_min_width_offset") (cVoid --> GSize.PolyML.cVal)
      val minHeight'Offset_ = call (getSymbol "giraffe_gdk_geometry_min_height_offset") (cVoid --> GSize.PolyML.cVal)
      val maxWidth'Offset_ = call (getSymbol "giraffe_gdk_geometry_max_width_offset") (cVoid --> GSize.PolyML.cVal)
      val maxHeight'Offset_ = call (getSymbol "giraffe_gdk_geometry_max_height_offset") (cVoid --> GSize.PolyML.cVal)
      val baseWidth'Offset_ = call (getSymbol "giraffe_gdk_geometry_base_width_offset") (cVoid --> GSize.PolyML.cVal)
      val baseHeight'Offset_ = call (getSymbol "giraffe_gdk_geometry_base_height_offset") (cVoid --> GSize.PolyML.cVal)
      val widthInc'Offset_ = call (getSymbol "giraffe_gdk_geometry_width_inc_offset") (cVoid --> GSize.PolyML.cVal)
      val heightInc'Offset_ = call (getSymbol "giraffe_gdk_geometry_height_inc_offset") (cVoid --> GSize.PolyML.cVal)
      val minAspect'Offset_ = call (getSymbol "giraffe_gdk_geometry_min_aspect_offset") (cVoid --> GSize.PolyML.cVal)
      val maxAspect'Offset_ = call (getSymbol "giraffe_gdk_geometry_max_aspect_offset") (cVoid --> GSize.PolyML.cVal)
      val winGravity'Offset_ = call (getSymbol "giraffe_gdk_geometry_win_gravity_offset") (cVoid --> GSize.PolyML.cVal)
    end
    type t = GdkGeometryRecord.t
    type gravity_t = GdkGravity.t
    structure MinWidthField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) minWidth'Offset_
      )
    structure MinHeightField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) minHeight'Offset_
      )
    structure MaxWidthField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) maxWidth'Offset_
      )
    structure MaxHeightField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) maxHeight'Offset_
      )
    structure BaseWidthField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) baseWidth'Offset_
      )
    structure BaseHeightField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) baseHeight'Offset_
      )
    structure WidthIncField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) widthInc'Offset_
      )
    structure HeightIncField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GInt)
        val offset = (I ---> GSize.FFI.fromVal) heightInc'Offset_
      )
    structure MinAspectField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GDouble)
        val offset = (I ---> GSize.FFI.fromVal) minAspect'Offset_
      )
    structure MaxAspectField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GDouble)
        val offset = (I ---> GSize.FFI.fromVal) maxAspect'Offset_
      )
    structure WinGravityField =
      Field(
        structure Container = GdkGeometryRecord
        structure FieldType = ValueFieldType(GdkGravity)
        val offset = (I ---> GSize.FFI.fromVal) winGravity'Offset_
      )
    val minWidth =
      {
        get = fn x => MinWidthField.get x,
        set = fn x => MinWidthField.set x
      }
    val minHeight =
      {
        get = fn x => MinHeightField.get x,
        set = fn x => MinHeightField.set x
      }
    val maxWidth =
      {
        get = fn x => MaxWidthField.get x,
        set = fn x => MaxWidthField.set x
      }
    val maxHeight =
      {
        get = fn x => MaxHeightField.get x,
        set = fn x => MaxHeightField.set x
      }
    val baseWidth =
      {
        get = fn x => BaseWidthField.get x,
        set = fn x => BaseWidthField.set x
      }
    val baseHeight =
      {
        get = fn x => BaseHeightField.get x,
        set = fn x => BaseHeightField.set x
      }
    val widthInc =
      {
        get = fn x => WidthIncField.get x,
        set = fn x => WidthIncField.set x
      }
    val heightInc =
      {
        get = fn x => HeightIncField.get x,
        set = fn x => HeightIncField.set x
      }
    val minAspect =
      {
        get = fn x => MinAspectField.get x,
        set = fn x => MinAspectField.set x
      }
    val maxAspect =
      {
        get = fn x => MaxAspectField.get x,
        set = fn x => MaxAspectField.set x
      }
    val winGravity =
      {
        get = fn x => WinGravityField.get x,
        set = fn x => WinGravityField.set x
      }
    fun new
      {
        minWidth,
        minHeight,
        maxWidth,
        maxHeight,
        baseWidth,
        baseHeight,
        widthInc,
        heightInc,
        minAspect,
        maxAspect,
        winGravity
      } =
      let
        fun init_ p =
          (
            MinWidthField.C.set minWidth p;
            MinHeightField.C.set minHeight p;
            MaxWidthField.C.set maxWidth p;
            MaxHeightField.C.set maxHeight p;
            BaseWidthField.C.set baseWidth p;
            BaseHeightField.C.set baseHeight p;
            WidthIncField.C.set widthInc p;
            HeightIncField.C.set heightInc p;
            MinAspectField.C.set minAspect p;
            MaxAspectField.C.set maxAspect p;
            WinGravityField.C.set winGravity p;
            ()
          )
        val retVal & () = (GdkGeometryRecord.FFI.withNewPtr ---> GdkGeometryRecord.FFI.fromPtr true && I) init_ ()
      in
        retVal
      end
  end
